import { SObjectObserver } from "./SObjectObserver";

/**
 * Sybotan 对象
 *
 * @author  庞利祥(sybotan@126.com)
 */
export class SObject {
    /** 对象名 */
    name: string = "";
    /** 监听数组 */
    private listeners = {};
    /** 对象id    */
    id: string = "";

    /**
     * 连接信号
     *
     * @param   name        信息名称
     * @param   receiver    信号接收者
     * @param   callback    回调函数
     */
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    connect(name: string, receiver: any, callback: Function): void {
        // @ts-ignore
        let observerList = this.listeners[name];
        if (!observerList) {
            // @ts-ignore
            this.listeners[name] = [];
        } else {
            // 如果已经连接，则返回
            for (let observer of observerList) {
                if (observer.compar(receiver)) {
                    return;
                }
            }
        }

        // @ts-ignore
        this.listeners[name].push(new SObjectObserver(receiver, callback));
    } // Function

    /**
     * 断开连接的信号
     *
     * @param   name        信息名称
     * @param   receiver    信号接收者
     */
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    disconnect(name: string, receiver: any): void {
        // @ts-ignore
        let observerList = this.listeners[name];
        if (!observerList) {
            return;
        }

        let length = observerList.length;
        for (let i = 0; i < length; i++) {
            let observer = observerList[i];
            if (observer.compar(receiver)) {
                observerList.splice(i, 1);
                break;
            }
        }
        if (observerList.length == 0) {
            // @ts-ignore
            delete this.listeners[name];
        }
    } // Function disconnect()

    /**
     * 发出事件
     *
     * @param   name    事件名称
     * @param   args    参数
     */
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    $emit(name: string, ...args: any): void {
        // @ts-ignore
        let observerList = this.listeners[name];
        if (!observerList) {
            return;
        }

        for (let observer of observerList) {
            observer.notify(this, args);
        }
    } // Function $emit()
} // Class SObject
