import { SParser } from "./SParser";
import { FloorData } from "../types/FloorData";
import { Wall } from "../types/floor/Wall";
import { Column } from "../types/floor/Column";
import { Casement } from "../types/floor/Casement";
import { VirtualWall } from "../types/floor/VirtualWall";
import { Door } from "../types/floor/Door";
import { Space } from "../types/floor/Space";
import { SWallItem } from "../items/floor/SWallItem";
import { SColumnItem } from "../items/floor/SColumnItem";
import { SDoorItem } from "../items/floor/SDoorItem";
import { SWindowItem } from "../items/floor/SWindowItem";
import { SVirtualWallItem } from "../items/floor/SVirtualWallItem";
import { SSpaceItem } from "../items/floor/SSpaceItem";

/**
 *  楼层信息解析器
 *
 */
export class SFloorParser extends SParser {
    /** 墙list   */
    wallList: SWallItem[] = [];
    /** 柱子list   */
    columnList: SColumnItem[] = [];
    /** 门list   */
    doorList: SDoorItem[] = [];
    /** 窗list   */
    casementList: SWindowItem[] = [];
    /** 虚拟墙list   */
    virtualWallList: SVirtualWallItem[] = [];
    /** 空间list   */
    spaceList: SSpaceItem[] = [];

    /**
     * 解析数据
     *
     * @param   data    楼层数据
     * */
    parseData(data: FloorData): void {
        if (data.Walls) {
            data.Walls.forEach((t: Wall): void => {
                this.addWall(t);
            });
        }
        if (data.Columns) {
            data.Columns.forEach((t: Column): void => {
                this.addColumn(t);
            });
        }
        if (data.Windows) {
            data.Windows.forEach((t: Casement): void => {
                this.addCasement(t);
            });
        }
        if (data.VirtualWalls) {
            data.VirtualWalls.forEach((t: VirtualWall): void => {
                this.addVirtualWall(t);
            });
        }
        if (data.Doors) {
            data.Doors.forEach((t: Door): void => {
                this.addDoor(t);
            });
        }
        if (data.Spaces) {
            data.Spaces.forEach((t: Space): void => {
                this.addSpace(t);
            });
        }
    } // Function parseData()

    /**
     * 添加墙至场景中
     *
     * @param   t       墙数据
     * */
    private addWall(t: Wall): void {
        let item = this.factory.createWall(t);
        this.wallList.push(item);
    } // Function addWall()

    /**
     * 添加柱子至场景中
     *
     * @param   t       柱子数据
     * */
    private addColumn(t: Column): void {
        let item = this.factory.createColumn(t);
        this.columnList.push(item);
    } // Function addColumn()

    /**
     * 添加窗户至场景中
     *
     * @param   t       窗户数据
     * */
    private addCasement(t: Casement): void {
        let item = this.factory.createWindow(t);
        this.casementList.push(item);
    } // Function addCasement()

    /**
     * 添加虚拟墙至场景中
     *
     * @param   t       虚拟墙数据
     * */
    private addVirtualWall(t: VirtualWall): void {
        let item = this.factory.createVirtualWall(t);
        this.virtualWallList.push(item);
    } // Function addVirtualWall()

    /**
     * 添加门至场景中
     *
     * @param   t       门数据
     * */
    private addDoor(t: Door): void {
        let item = this.factory.createDoor(t);
        this.doorList.push(item);
    } // Function addDoor()

    /**
     * 添加空间至场景中
     *
     * @param   t       空间数据
     * */
    private addSpace(t: Space): void {
        let item = this.factory.createSpace(t);
        this.spaceList.push(item);
    } // Function addSpace()
} // class SFloorParser
