/**
 * 点数据类型定义
 *
 * @author 庞利祥(sybotan@126.com)
 */
import { SMatrix } from "@saga-web/base/lib";

export class SPoint {
    ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    // 属性定义
    /** X轴坐标 */
    x: number;
    /** Y轴坐标 */
    y: number;

    ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    // 构造函数
    /**
     * 构造函数
     */
    constructor();

    /**
     * 构造函数
     *
     * @param   p       点
     */
    constructor(p: SPoint);

    /**
     * 构造函数
     *
     * @param   x       X轴坐标
     * @param   y       Y轴坐标
     */
    constructor(x: number, y: number);

    /**
     * 构造函数(重载实现)
     *
     * @param   x       X轴坐标 | 点 | undefined
     * @param   y       Y轴坐标
     */
    constructor(x?: number | SPoint, y?: number) {
        if (x == undefined) {
            this.x = 0;
            this.y = 0;
        } else if (x instanceof SPoint) {
            this.x = x.x;
            this.y = x.y;
        } else {
            this.x = x as number;
            this.y = y as number;
        }
    } // Function constructor()

    /**
     * 设置点
     * @param   pos      点
     */
    setPoint(pos: SPoint): void;

    /**
     * 设置点
     * @param   x       X轴坐标
     * @param   y       Y轴坐标
     */
    setPoint(x: number, y: number): void;

    /**
     * 设置点(重载实现)
     * @param   x       X轴坐标 | 点
     * @param   y       Y轴坐标
     */
    setPoint(x: number | SPoint, y?: number): void {
        if (x instanceof SPoint) {
            this.x = x.x;
            this.y = x.y;
        } else {
            this.x = x as number;
            this.y = y as number;
        }
    } // Function setPoint()

    /**
     * 布线长度
     *
     * @return  返回 |X| + |Y|
     */
    manhattanLength(): number {
        // noinspection JSSuspiciousNameCombination
        return Math.abs(this.x) + Math.abs(this.y);
    } // Function manhattanLength()

    /**
     * 返回点矩阵变换后的值
     *
     * @return  变换后的点
     * */
    matrixTransform(mat: SMatrix): SPoint {
        const p = new SPoint();
        p.x = this.x * mat.a + this.y * mat.b + mat.e;
        p.y = this.x * mat.c + this.y * mat.d + mat.f;
        return p;
    } // Function matrixTransform()

    /**
     * 平移点
     *
     * @param   dx      X轴位移
     * @param   dy      Y轴位移
     */
    translate(dx: number, dy: number): void {
        this.x += dx;
        this.y += dy;
    } // Function translate()
} // Class SPoint
