import {SAspectRatioMode} from "..";

/**
 * 大小数据类型定义
 *
 * @author 庞利祥(sybotan@126.com)
 */
export class SSize {
    ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    // 属性定义
    /** 宽度 */
    width: number;
    /** 高度 */
    height: number;

    ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    // 构造函数
    /**
     * 构造函数
     */
    constructor();

    /**
     * 构造函数
     *
     * @param   size    大小
     */
    constructor(size: SSize);

    /**
     * 构造函数(重载实现)
     *
     * @param   width   宽度
     * @param   height  高度
     */
    constructor(width: number, height: number);

    /**
     * 构造函数(重载实现)
     *
     * @param   width   宽度
     * @param   height  高度
     */
    constructor(width?: number | SSize, height?: number) {
        if (width == undefined) {
            this.width = 0;
            this.height = 0;
        } else if (width instanceof SSize) {
            this.width = width.width;
            this.height = width.height;
        } else {
            this.width = width as number;
            this.height = height as number;
        }
    } // Function constructor()

    ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    // 方法
    /**
     * 是否为空
     *
     * @return  width或height小于等于0，返回true，否则返回false。
     *
     * @see isNull(), isValid()
     */
    isEmpty(): boolean {
        return this.width <= 0 || this.height <= 0;
    } // Function isEmpty()

    /**
     * 是否为Null
     *
     * @return  width与height都等于0，返回true，否则返回false。
     *
     * @see isEmpty(), isValid()
     */
    isNull(): boolean {
        return this.width == 0 && this.height == 0;
    } // Function isNull()

    /**
     * 是否有效
     *
     * @return  width与height都大于0，返回true，否则返回false。
     * @see isEmpty(), isNull()
     */
    isValid(): boolean {
        return this.width > 0 && this.height > 0;
    } // Function isValid()

    /**
     * 缩放
     *
     * @param   width       宽度
     * @param   height      高度
     * @param   mode        缩放模式
     */
    scale(
        width: number,
        height: number,
        mode: SAspectRatioMode = SAspectRatioMode.IgnoreAspectRatio
    ): void {
        let s = this.scaled(width, height, mode);
        this.width = s.width;
        this.height = s.height;
    } // Function scale()

    /**
     * 生成缩放大小
     *
     * @param   width       宽度
     * @param   height      高度
     * @param   mode        缩放模式
     * @return  缩放大小
     */
    scaled(
        width: number,
        height: number,
        mode: SAspectRatioMode = SAspectRatioMode.IgnoreAspectRatio
    ): SSize {
        let w = 0;
        let h = 0;
        if (mode == SAspectRatioMode.KeepAspectRatio) {
            let s = Math.min(width / this.width, height / this.height);
            w = this.width * s;
            h = this.height * s;
        } else if (mode == SAspectRatioMode.KeepAspectRatioByExpanding) {
            let s = Math.max(width / this.width, height / this.height);
            w = this.width * s;
            h = this.height * s;
        } else {
            // 自由模式
            w = width;
            h = height;
        }
        return new SSize(w, h);
    } // Function scaled()

    /**
     * 互换width与height
     *
     * @see transposed()
     */
    transpose(): void {
        let size = this.transposed();
        this.width = size.width;
        this.height = size.height;
    } // Function transpose()

    /**
     * 生成互换width与height的SSize
     *
     * @return  返回互换width与height的SSize。
     *
     * @see transpose()
     */
    transposed(): SSize {
        return new SSize(this.height, this.width);
    } // Function transposed()
} // Class SSize
