import { SGraphCommand } from "./SGraphCommand";
import { SGraphItem } from "../SGraphItem";
import { SPoint } from "@saga-web/draw/lib";
import { SUndoCommand } from "@saga-web/base/lib";
import { SGraphScene } from "../SGraphScene";

/**
 * item移动命令
 *
 * @author  panglixiang
 *
 * */
export class SGraphMoveCommand extends SGraphCommand {
    /** 命令名称    */
    readonly command: string;
    /** 执行命令的item   */
    item: SGraphItem;
    /** 移动前位置    */
    old: SPoint;
    /** 移动后位置   */
    pos: SPoint;

    /**
     * 构造函数
     *
     * @param   scene   当前场景
     * @param   item    item对象
     * @param   old     移动前位置
     * @param   pos     移动后位置
     * */
    constructor(
        scene: SGraphScene,
        item: SGraphItem,
        old: SPoint,
        pos: SPoint
    ) {
        super(scene);
        this.item = item;
        this.old = old;
        this.pos = pos;
        this.command = "SGraphMoveCommand";
        this.desc = `移动对象=${item.id}`;
    } // Constructor

    /**
     * 合并命令
     *
     * @param   command 命令
     * @return  boolean 是否已执行合并命令
     * */
    mergeWith(command: SUndoCommand): boolean {
        if (command instanceof SGraphMoveCommand && command.item == this.item) {
            command.pos = this.pos;
            return true;
        }
        return false;
    } // Function mergeWith()

    /**
     * 重做
     *
     * */
    redo(): void {
        this.item.pos = new SPoint(this.pos.x, this.pos.y);
        this.item.update();
    } // Function redo()

    /**
     * 撤销
     *
     * */
    undo(): void {
        this.item.pos = new SPoint(this.old.x, this.old.y);
        this.item.update();
    } // Function undo()

    /**
     * 命令细节描述
     *
     * */
    toString(): string {
        return `oldPos=${JSON.stringify(this.old)};\nnewPos=${JSON.stringify(
            this.pos
        )}`;
    } // Function toString()
} // Class SGraphMoveCommand
