import { SGraphCommand } from "./SGraphCommand";
import { SGraphItem } from "../SGraphItem";
import { SGraphScene } from "../SGraphScene";

/**
 * 属性修改命令类
 *
 * @author  张宇(taohuzy@163.com)
 */

export class SGraphPropertyCommand<T> extends SGraphCommand {
    /** 命令名称    */
    readonly command: string;
    /**	指向item对象	*/
    item: SGraphItem;
    /** 属性名称 */
    propName: string;
    /** 属性旧值 */
    oldProp: T;
    /** 属性新值 */
    newProp: T;

    /**
     * 构造函数
     *
     * @param   scene       命令所属的场景类
     * @param   item        命令所属的item类
     * @param   propName    修改的属性名称
     * @param   oldProp     修改前的属性值
     * @param   newProp     修改后的属性值
     */
    constructor(
        scene: SGraphScene,
        item: SGraphItem,
        propName: string,
        oldProp: T,
        newProp: T
    ) {
        super(scene);
        this.item = item;
        this.propName = propName;
        this.oldProp = oldProp;
        this.newProp = newProp;
        this.command = "SGraphPropertyCommand";
        this.command = `更新属性=${item.id}`;
    } // Constructor

    /**
     * redo操作
     */
    redo(): void {
        // @ts-ignore
        this.item[this.propName] = this.newProp;
        this.item.update();
    } // Function redo()

    /**
     * undo操作
     */
    undo(): void {
        // @ts-ignore
        this.item[this.propName] = this.oldProp;
        this.item.update();
    } // Function undo()

    /**
     * 命令细节描述
     *
     * */
    toString(): string {
        const propName = `propName=${this.propName}`;
        const oldProp = `oldProp=${JSON.stringify(this.oldProp)}`;
        const newProp = `newProp=${JSON.stringify(this.newProp)}`;
        return `${propName};\n${oldProp};\n${newProp}`;
    } // Function toString()
} // Class SGraphPropertyCommand
