import { SPainter, SColor } from "@saga-web/draw/lib";
import { SGraphItem } from "../SGraphItem";
import { v1 as uuidv1 } from "uuid";

/**
 * 锚点item
 *
 * @author  郝建龙(1061851420@qq.com)
 */
export class SAnchorItem extends SGraphItem {
    /** 锚点宽 */
    private width: number = 3;
    /** 锚点高 */
    private height: number = 3;
    /** 是否被连接   */
    isConnected: boolean = false;
    /** 全局灵敏度   */
    dis: number = 3;
    /** 灵敏度转换为场景长度  */
    sceneDis: number = 3;

    /**
     * 构造函数
     *
     * @param   parent      指向父对象
     */
    constructor(parent: SGraphItem | null = null) {
        super(parent);
        this.id = uuidv1();
    } // Function constructor()

    /**
     * Item绘制操作
     *
     * @param   painter       painter对象
     */
    onDraw(painter: SPainter): void {
        this.sceneDis = painter.toPx(this.dis);
        painter.pen.lineWidth = painter.toPx(1);
        painter.pen.color = new SColor("#2196f3");
        painter.brush.color = SColor.White;
        if (this.isConnected) {
            painter.brush.color = painter.pen.color;
        }
        painter.drawRect(
            -this.width / 2,
            -this.height / 2,
            this.width,
            this.height
        );
    } // Function onDraw()
} // Class SAnchorItem
