import { SColor, SPainter, SPen, SRect } from "@saga-web/draw/lib";
import { SGraphyItem } from "../SGraphyItem";

/**
 * 线Item类
 *
 * @author  庞利祥(sybotan@126.com)
 */
export class SGraphyLineItem extends SGraphyItem {
    x1: number;
    y1: number;
    x2: number;
    y2: number;
    color: SColor = SColor.Black;
    width: number = 1;

    /**
     * 构造函数
     *
     * @param x1  线的起始x坐标
     * @param y1  线的起始y坐标
     * @param x2    线的终止x坐标
     * @param y2    线的终止y坐标
     * @param width   线的宽度
     *
     * @param color  线的颜色
     * @param parent    是否为虚线
     */
    constructor(
        parent: SGraphyItem | null,
        x1: number,
        y1: number,
        x2: number,
        y2: number,
        color: SColor = SColor.Black,
        width: number = 1
    ) {
        super(parent);
        this.x1 = x1;
        this.y1 = y1;
        this.x2 = x2;
        this.y2 = y2;
    } // Constructor()

    /**
     * Item对象边界区域
     *
     * @return SRect
     */
    boundingRect(): SRect {
        let minX = Math.min(this.x1, this.x2);
        let minY = Math.min(this.y1, this.y2);
        let maxX = Math.max(this.x1, this.x2);
        let maxY = Math.max(this.y1, this.y2);
        return new SRect(
            minX - this.width / 2,
            minY - this.width / 2,
            maxX - minX + this.width,
            maxY - minY + this.width
        );
    } // Function boundingRect()

    /**
     * 绘制Item
     *
     * @param   painter     画布
     */
    onDraw(painter: SPainter): void {
        painter.pen = new SPen(this.color, this.width);
        painter.drawLine(this.x1, this.y1, this.x2, this.y2);
    } // Function onDraw()
} // Class SGraphyLineItem
